/*
 * Copyright 2010 Tilburg University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package nl.uvt.pdproxy;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import nl.uvt.commons.io.*;
import nu.xom.*;
import nu.xom.xslt.*;

public class ExpertsAndExpertiseSource implements ISource {

  private String urlTemplate;
  private int idPos;
  private XSLTransform xslTransform;
  
  public ExpertsAndExpertiseSource(String urlTemplate, Document xslDocument)
  throws ConfigurationException, IOException, ValidityException, ParsingException, XSLException {
    this.urlTemplate = urlTemplate;
    this.idPos = this.urlTemplate.indexOf("$id");
    if (this.idPos < 0) {
      throw new ConfigurationException("Missing string '$id' in URL template '"+urlTemplate+"'");
    }
    this.xslTransform = new XSLTransform(xslDocument);
  }
  
  public Person getPerson(String id)
  throws UnknownObjectException, SourceException {
    Person person = null;
    try {
      URL url = this.queryURL(id);
      Document expertsAndExpertiseDocument = Http.httpGetDocument(url, null);
      Element rootElement = expertsAndExpertiseDocument.getRootElement();
      if (rootElement.getLocalName().equals("error")) {
        if (rootElement.getValue().equals("Invalid parameters")) {
          throw new UnknownObjectException(
              "Experts and Expertise returned 'Invalid parameters' error; "+
              "usually this is because a person is unknown");
          // TODO: must ask for a more explicit error message from E&E if person unknown
        }
        else {
          throw new SourceException("Experts and Expertise returned error. Msg="+rootElement.getValue());
        }
      }
      Document foafDocument = this.transformExpertsAndExpertiseResponseToFOAF(expertsAndExpertiseDocument);
      person = new Person(id, foafDocument);
    } catch (MalformedURLException e) {
      throw new SourceException("MalformedURLException occurred. Msg="+e.getMessage());
    } catch (UnsupportedEncodingException e) {
      throw new SourceException("UnsupportedEncodingException occurred. Msg="+e.getMessage());
    } catch (ParsingException e) {
      throw new SourceException("ParsingException occurred. Msg="+e.getMessage());
    } catch (XSLException e) {
      throw new SourceException("XSLException occurred. Msg="+e.getMessage());
    } catch (IOException e) {
      throw new SourceException("IOException occurred. Msg="+e.getMessage());
    }
    return person;
  }

  /**
   * Returns a full URL for querying Experts and Expertise.
   * 
   * @param id the person identifier
   * @return the URL for querying Experts and Expertise
   * @throws MalformedURLException
   */
  private URL queryURL(String id)
  throws UnsupportedEncodingException, MalformedURLException {
    URL url = null;
    StringBuffer request = new StringBuffer(this.urlTemplate);
    String idEncoded;
    idEncoded = URLEncoder.encode(id, "UTF-8");
    request.replace(this.idPos, this.idPos+3, idEncoded);
    url = new URL(request.toString());
    return url;
  }
  
  /**
   * Transforms the Experts and Expertise response into an FOAF document.
   * 
   * @param response the response from Experts and Expertise
   * @return an FOAF document
   */
  public Document transformExpertsAndExpertiseResponseToFOAF(Document response)
  throws XSLException {
    Document foafDocument = null;
    Nodes transformedNodes = this.xslTransform.transform(response);
    foafDocument = XSLTransform.toDocument(transformedNodes);
    return foafDocument;
  }
}
